#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "peri.h"
#include "usbdrv.h"
#define RED1    PC1
#define RED2    PC5
#define GREEN   PC3
#define Ldr_RED1    PC0
#define Ldr_RED2    PC4
#define Ldr_GREEN   PC2
#define ON     1
#define OFF    0
#define RQ_GET_SWITCH_1      1
#define RQ_GET_SWITCH_2      2
#define RQ_GET_SWITCH_3      3
#define RQ_GET_SWITCH_4      4
#define RQ_GET_SWITCH_5      5
#define RQ_GET_LIGHT_1       6
#define RQ_GET_LIGHT_2       7
#define RQ_GET_LIGHT_3       8

uint8_t light_state1 = 1; 
uint8_t light_state2 = 1;
uint8_t light_state3 = 1; 

int main()
{
    init_peripheral();
    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    uint16_t Red1;
    uint16_t Red2;
    uint16_t Green;
    
    while (1)
    {	
	usbPoll();
	Red1=read_adc(Ldr_RED1);
	Red2=read_adc(Ldr_RED2);
	Green=read_adc(Ldr_GREEN);
	
        if(Red1<500){
		set_led(RED1,OFF);
		light_state1 = 0;
	}
      else{
        	set_led(RED1,ON);
		light_state1 = 1;
	}
	if(Red2<500){
		set_led(RED2,OFF);
		light_state3 = 0;
	}
      	else{
        	set_led(RED2,ON);
		light_state3 = 1;
	}
	if(Green<500){
		set_led(GREEN,OFF);
		light_state2 = 0; 
	}
     	else{
        	set_led(GREEN,ON);
		light_state2 = 1; 
	}
    }

    return 0;
}

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    /* declared as static so they stay valid when usbFunctionSetup returns */
    static uint8_t switch_state; 
    

     if (rq->bRequest == RQ_GET_SWITCH_1)
    {
        switch_state = IS_SWITCH1_PRESSED();
	

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }else if (rq->bRequest == RQ_GET_SWITCH_2)
    {
        switch_state = IS_SWITCH2_PRESSED();
	

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }else if (rq->bRequest == RQ_GET_SWITCH_3){
    
        switch_state = IS_SWITCH3_PRESSED();
	

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }else if (rq->bRequest == RQ_GET_SWITCH_4){
    
        switch_state = IS_SWITCH4_PRESSED();
	

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }else if (rq->bRequest == RQ_GET_SWITCH_5){
    
        switch_state = IS_SWITCH5_PRESSED();
	

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }
    else if (rq->bRequest == RQ_GET_LIGHT_1){	

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &light_state1;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }
    else if (rq->bRequest == RQ_GET_LIGHT_2){	

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &light_state2;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }
    else if (rq->bRequest == RQ_GET_LIGHT_3){	

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &light_state3;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }

    /* default for not implemented requests: return no data back to host */
    return 0;
}

